@@ -1,20 +1,19 @@
-from http import HTTPStatus
 import json
 import os
 import shutil
 from datetime import datetime
+from http import HTTPStatus
 from pathlib import Path
 from typing import Dict, List, Optional, Union, cast
 
-from fps.hooks import register_router  # type: ignore
-from fastapi import APIRouter, Depends, HTTPException, Response
-from starlette.requests import Request  # type: ignore
 from anyio import open_file
+from fastapi import APIRouter, Depends, HTTPException, Response
+from fps.hooks import register_router  # type: ignore
+from starlette.requests import Request  # type: ignore
 
-from fps_auth.backends import current_user  # type: ignore
-from fps_auth.models import User  # type: ignore
+from jupyverse import User, current_user
 
-from .models import Checkpoint, Content, SaveContent, CreateContent, RenameContent
+from .models import Checkpoint, Content, CreateContent, RenameContent, SaveContent
 
 router = APIRouter()
 
@@ -23,11 +22,11 @@ router = APIRouter()
     "/api/contents/{path:path}/checkpoints",
     status_code=201,
 )
-async def create_checkpoint(path, user: User = Depends(current_user)):
+async def create_checkpoint(
+    path, user: User = Depends(current_user(permissions={"contents": ["write"]}))
+):
     src_path = Path(path)
-    dst_path = (
-        Path(".ipynb_checkpoints") / f"{src_path.stem}-checkpoint{src_path.suffix}"
-    )
+    dst_path = Path(".ipynb_checkpoints") / f"{src_path.stem}-checkpoint{src_path.suffix}"
     try:
         dst_path.parent.mkdir(exist_ok=True)
         shutil.copyfile(src_path, dst_path)
@@ -45,7 +44,7 @@ async def create_checkpoint(path, user: User = Depends(current_user)):
 async def create_content(
     path: Optional[str],
     request: Request,
-    user: User = Depends(current_user),
+    user: User = Depends(current_user(permissions={"contents": ["write"]})),
 ):
     create_content = CreateContent(**(await request.json()))
     content_path = Path(create_content.path)
@@ -53,14 +52,10 @@ async def create_content(
         available_path = get_available_path(content_path / "Untitled.ipynb")
         async with await open_file(available_path, "w") as f:
             await f.write(
-                json.dumps(
-                    {"cells": [], "metadata": {}, "nbformat": 4, "nbformat_minor": 5}
-                )
+                json.dumps({"cells": [], "metadata": {}, "nbformat": 4, "nbformat_minor": 5})
             )
         src_path = available_path
-        dst_path = (
-            Path(".ipynb_checkpoints") / f"{src_path.stem}-checkpoint{src_path.suffix}"
-        )
+        dst_path = Path(".ipynb_checkpoints") / f"{src_path.stem}-checkpoint{src_path.suffix}"
         try:
             dst_path.parent.mkdir(exist_ok=True)
             shutil.copyfile(src_path, dst_path)
@@ -73,28 +68,26 @@ async def create_content(
         available_path.mkdir(parents=True, exist_ok=True)
     else:
         assert create_content.ext is not None
-        available_path = get_available_path(
-            content_path / ("untitled" + create_content.ext)
-        )
+        available_path = get_available_path(content_path / ("untitled" + create_content.ext))
         open(available_path, "w").close()
 
-    return Content(**await get_path_content(available_path, False))
+    return await read_content(available_path, False)
 
 
 @router.get("/api/contents")
 async def get_root_content(
     content: int,
-    user: User = Depends(current_user),
+    user: User = Depends(current_user(permissions={"contents": ["read"]})),
 ):
-    return Content(**await get_path_content(Path(""), bool(content)))
+    return await read_content("", bool(content))
 
 
 @router.get("/api/contents/{path:path}/checkpoints")
-async def get_checkpoint(path, user: User = Depends(current_user)):
+async def get_checkpoint(
+    path, user: User = Depends(current_user(permissions={"contents": ["read"]}))
+):
     src_path = Path(path)
-    dst_path = (
-        Path(".ipynb_checkpoints") / f"{src_path.stem}-checkpoint{src_path.suffix}"
-    )
+    dst_path = Path(".ipynb_checkpoints") / f"{src_path.stem}-checkpoint{src_path.suffix}"
     if not dst_path.exists():
         return []
     mtime = get_file_modification_time(dst_path)
@@ -104,37 +97,25 @@ async def get_checkpoint(path, user: User = Depends(current_user)):
 @router.get("/api/contents/{path:path}")
 async def get_content(
     path: str,
-    content: int,
-    user: User = Depends(current_user),
+    content: int = 0,
+    user: User = Depends(current_user(permissions={"contents": ["read"]})),
 ):
-    return Content(**await get_path_content(Path(path), bool(content)))
+    return await read_content(path, bool(content))
 
 
 @router.put("/api/contents/{path:path}")
 async def save_content(
+    path,
     request: Request,
-    user: User = Depends(current_user),
+    response: Response,
+    user: User = Depends(current_user(permissions={"contents": ["write"]})),
 ):
-    save_content = SaveContent(**(await request.json()))
+    content = SaveContent(**(await request.json()))
     try:
-        async with await open_file(save_content.path, "w") as f:
-            if save_content.format == "json":
-                dict_content = cast(Dict, save_content.content)
-                if save_content.type == "notebook":
-                    # see https://github.com/jupyterlab/jupyterlab/issues/11005
-                    if (
-                        "metadata" in dict_content
-                        and "orig_nbformat" in dict_content["metadata"]
-                    ):
-                        del dict_content["metadata"]["orig_nbformat"]
-                    await f.write(json.dumps(dict_content, indent=2))
-            else:
-                str_content = cast(str, save_content.content)
-                await f.write(str_content)
+        await write_content(content)
     except Exception:
-        # FIXME: return error code?
-        pass
-    return Content(**await get_path_content(Path(save_content.path), False))
+        raise HTTPException(status_code=404, detail=f"Error saving {content.path}")
+    return await read_content(content.path, False)
 
 
 @router.delete(
@@ -143,7 +124,7 @@ async def save_content(
 )
 async def delete_content(
     path,
-    user: User = Depends(current_user),
+    user: User = Depends(current_user(permissions={"contents": ["write"]})),
 ):
     p = Path(path)
     if p.exists():
@@ -158,11 +139,11 @@ async def delete_content(
 async def rename_content(
     path,
     request: Request,
-    user: User = Depends(current_user),
+    user: User = Depends(current_user(permissions={"contents": ["write"]})),
 ):
     rename_content = RenameContent(**(await request.json()))
     Path(path).rename(rename_content.path)
-    return Content(**await get_path_content(Path(rename_content.path), False))
+    return await read_content(rename_content.path, False)
 
 
 def get_file_modification_time(path: Path):
@@ -178,7 +159,7 @@ def get_file_creation_time(path: Path):
 def get_file_size(path: Path) -> Optional[int]:
     if path.exists():
         return path.stat().st_size
-    return None
+    raise HTTPException(status_code=404, detail="Item not found")
 
 
 def is_file_writable(path: Path) -> bool:
@@ -191,12 +172,14 @@ def is_file_writable(path: Path) -> bool:
     return False
 
 
-async def get_path_content(path: Path, get_content: bool):
-    content: Optional[Union[str, List[Dict]]] = None
+async def read_content(path: Union[str, Path], get_content: bool, as_json: bool = False) -> Content:
+    if isinstance(path, str):
+        path = Path(path)
+    content: Optional[Union[str, Dict, List[Dict]]] = None
     if get_content:
         if path.is_dir():
             content = [
-                await get_path_content(subpath, get_content=False)
+                (await read_content(subpath, get_content=False)).dict()
                 for subpath in path.iterdir()
                 if not subpath.name.startswith(".")
             ]
@@ -204,6 +187,8 @@ async def get_path_content(path: Path, get_content: bool):
             try:
                 async with await open_file(path) as f:
                     content = await f.read()
+                if as_json:
+                    content = json.loads(content)
             except Exception:
                 raise HTTPException(status_code=404, detail="Item not found")
     format: Optional[str] = None
@@ -218,6 +203,20 @@ async def get_path_content(path: Path, get_content: bool):
             type = "notebook"
             format = None
             mimetype = None
+            if content is not None:
+                nb: dict
+                if as_json:
+                    content = cast(Dict, content)
+                    nb = content
+                else:
+                    content = cast(str, content)
+                    nb = json.loads(content)
+                for cell in nb["cells"]:
+                    if "metadata" not in cell:
+                        cell["metadata"] = {}
+                    cell["metadata"].update({"trusted": False})
+                if not as_json:
+                    content = json.dumps(nb)
         elif path.suffix == ".json":
             type = "json"
             format = "text"
@@ -229,18 +228,36 @@ async def get_path_content(path: Path, get_content: bool):
     else:
         raise HTTPException(status_code=404, detail="Item not found")
 
-    return {
-        "name": path.name,
-        "path": str(path),
-        "last_modified": get_file_modification_time(path),
-        "created": get_file_creation_time(path),
-        "content": content,
-        "format": format,
-        "mimetype": mimetype,
-        "size": size,
-        "writable": is_file_writable(path),
-        "type": type,
-    }
+    return Content(
+        **{
+            "name": path.name,
+            "path": str(path),
+            "last_modified": get_file_modification_time(path),
+            "created": get_file_creation_time(path),
+            "content": content,
+            "format": format,
+            "mimetype": mimetype,
+            "size": size,
+            "writable": is_file_writable(path),
+            "type": type,
+        }
+    )
+
+
+async def write_content(content: Union[SaveContent, Dict]) -> None:
+    if not isinstance(content, SaveContent):
+        content = SaveContent(**content)
+    async with await open_file(content.path, "w") as f:
+        if content.format == "json":
+            dict_content = cast(Dict, content.content)
+            if content.type == "notebook":
+                # see https://github.com/jupyterlab/jupyterlab/issues/11005
+                if "metadata" in dict_content and "orig_nbformat" in dict_content["metadata"]:
+                    del dict_content["metadata"]["orig_nbformat"]
+            await f.write(json.dumps(dict_content, indent=2))
+        else:
+            content.content = cast(str, content.content)
+            await f.write(content.content)
 
 
 def get_available_path(path: Path, sep: str = ""):
